@@ -5,15 +5,53 @@ from collections import defaultdict
 class Document:
     def __init__(self, xmlfile, pos=True, lemma=True, ner=True, parse=True,
                  coref=True, basic_deps=False, coll_deps=False,
-                 coll_ccp_deps=True):
+                 coll_ccp_deps=True, verbose=False):
 
         sents, coref_chains = _parse_source(xmlfile, use_pos=pos,
                                             use_lemma=lemma, use_ner=ner,
                                             use_parse=parse, use_coref=coref,
                                             use_basic_deps=basic_deps,
                                             use_coll_deps=coll_deps,
-                                            use_coll_ccp_deps=coll_ccp_deps)
+                                            use_coll_ccp_deps=coll_ccp_deps,
+                                            verbose=verbose)
         self.sents = sents
+        self.coref_map = self._build_coref_map(coref_chains)
+        self.mention_chains = set(self.coref_map.values())
+
+    def rep_head(self, token):
+        """
+        Get the representative mention head token for token.
+        If token is not part of a mention chain, then return the
+        token.
+
+        token -- A Token object.
+        """
+        mention_chain = self.coref_map.get(token, None)
+        if mention_chain is None:
+            return token
+        else:
+            return mention_chain.rep_head
+    
+    def mention_chain(self, token):
+        """
+        Return the corresponding mention chain for a token. If token
+        is not part of a mention chain, return None.
+
+        token - A Token object.
+        """
+        return self.coref_map.get(token, None)
+
+    def _build_coref_map(self, coref_chains):
+        coref_map = {}
+        for chain in coref_chains:
+            entity = MentionChain(chain, self)
+            for mention in chain:
+                sent = mention.sent
+                start = mention.start
+                end = mention.end
+                for token in self[sent][start:end]:
+                    coref_map[token] = entity 
+        return coref_map
 
     def __len__(self):
         return len(self.sents)
@@ -29,7 +67,9 @@ class Document:
         for t in self.sents[sent].tokens[start:end]:
             tokens.append(t._surface)
         return u' '.join(tokens)
-        
+
+    def __str__(self):
+        return u'\n'.join([unicode(s) for s in self.sents])        
 
 class Sentence:
     def __init__(self, tokens, parse,
@@ -157,16 +197,27 @@ class Mention:
                                                                     self.head,
                                                                     self.end)
 
+class MentionChain:
+    def __init__(self, mention_chain, doc):
+        mention_heads = []
+        mention_tokens = []                
+        for mention in mention_chain:
+            s = mention_chain[0].sent
+            h = mention_chain[0].head
+            start = mention_chain[0].start
+            end = mention_chain[0].end
+            mention_heads.append(doc[s][h])
+            mention_tokens.append(doc[s][start:end])
+        self.rep_head = mention_heads[0]
+        self.rep_tokens = mention_tokens[0]
+        self.mention_heads = mention_heads
+        self.mention_tokens = mention_tokens
 
 
 def _parse_source(source, use_pos=True, use_lemma=True, use_ner=True,
                   use_parse=True, use_coref=True, use_basic_deps=False,
-                  use_coll_deps=False, use_coll_ccp_deps=True):
+                  use_coll_deps=False, use_coll_ccp_deps=True, verbose=False):
 
-    if use_coref:
-        import sys
-        sys.stderr.write('Coref not implemented.\n')
-        sys.stderr.flush()
 
     # Temporary vars for token level attributes.
     _word = None
